<?php
namespace Teknisa\SUP\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\SUP\Modules\Cadastros\Services\FilialAtendidaNaoAtendidaPeloFornecedorService;

class FilialAtendidaNaoAtendidaPeloFornecedorController extends \Zeedhi\Framework\Controller\Report{

    protected $connection;
    protected $stNomeRel;
    protected $stExtensao;
    protected $birtStrategy;
    protected $qr2Strategy;
    
    public function __construct(ReportService $reportService, FilialAtendidaNaoAtendidaPeloFornecedorService $service, $connection, $birtStrategy,$qr2Strategy ) {
        parent::__construct($reportService);
        $this->connection    = $connection;
        $this->service       = $service;
        $this->birtStrategy  = $birtStrategy;
        $this->qr2Strategy   = $qr2Strategy;
    }
    
    public function concatArray($array){
        return join('_', $array);
    }
    
    public function saveCodPeri(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
       
        $codPeri = $this->service->findCodPeri($row);
        $result = $this->service->verificaPrazo($row);
        if($codPeri['CDPERIENTR'] != null){
            $this->service->updateCodPeri($row);
        }else{
            $this->service->insertCodPeri($row);    
        }
       
        if( $result[0]['PERMITE']=='S'){
            if($row['NRPRAZOFORNEXPEDFIL']==NULL && ($row['QTDIASPRAZOEXPED']!=NULL ||$row['QTDIASPRAZOEXPED']===0 )){
              
                 $row['NRPRAZOFORNEXPEDFIL']=$this->service->retornaProxprazo($row)[0]['PROXPRAZO'];
                 $this->service->insertPrazo($row);
            }else if($row['NRPRAZOFORNEXPEDFIL']!=NULL &&( $row['QTDIASPRAZOEXPED']===NULL)){
                  
                  $this->service->deletePrazo($row);
                  
            }else {
                      
               $this->service->updatePrazo($row); 
            }
        }
    }
    
    public function saveFilial(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $filialExists = false;
        
        $filial = $this->service->findFilial($row);
        //
        if(!empty($filial)){
            foreach ($filial as $value) {
                if($value['CDFILIAL'] == $row['CDFILIAL']){
                    $filialExists = true;
                    break;
                }
            }
        }
        
        if($filialExists == true){
            $erro = new Error("Operação bloqueada! A unidade informada já está cadastrada!", 500);
            $response->setError($erro);
        }else{
            $this->service->insertFilial($row);    
        }
    }
    
    public function filialDelete(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $result = $this->service->findFilialRegiaoNaoAtendida($row);
            
            if(!empty($result)){
                 throw new \Exception("Operação bloqueada. A região da unidade não é atendida pelo fornecedor.");
            }
            
            if(!empty($row)){
                foreach ($row as $value) {
                    $this->service->deleteFilial($value);
                }
            }
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deleteUnidadeAtendida(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        if($row['CDPERIENTR']){
            $params = [
                "CDPERIENTR"     => NULL,
                "CDFORNECED" => $row['CDFORNECED'],
                "CDFILIAL"   => $row['CDFILIAL']
            ];
            
            $query = "UPDATE REFORFIL SET CDPERIENTR = :CDPERIENTR WHERE CDFORNECED = :CDFORNECED AND CDFILIAL = :CDFILIAL";
        }else{
            $params = [
                "CDFORNECED"   => $row['CDFORNECED'],
                "CDFILIAL"     => $row['CDFILIAL'],
                "NRORG"        => $row['NRORG'],
            ];
            
            $query = "INSERT INTO REFORFIL (CDFORNECED, CDFILIAL, NRORG) VALUES (:CDFORNECED, :CDFILIAL, :NRORG)";
        }
        
        try{
            $std = $this->connection->prepare($query);
            $std->execute($params);
            if($row['NRPRAZOFORNEXPEDFIL']!=null){
                   $this->service->deletePrazo($row);
            }
        }catch(Exception $e){
            throw $e;
        }
    }
    
    
    
    public function exportFilial(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        
        $row['CDFORNECEDDEST'] = is_array($row['CDFORNECEDDEST']) ? count($row['CDFORNECEDDEST']) > 0 ? self::concatArray($row['CDFORNECEDDEST']) : 'T' : 'T';
        $row['CDFILIAL']       = is_array($row['CDFILIAL'])       ? count($row['CDFILIAL'])       > 0 ? self::concatArray($row['CDFILIAL'])       : 'T' : 'T';
        
        $this->service->insertExportation($row);
    }
    
    protected $reportMapping = array ();
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        
        $this->stNomeRel = $this->getReportName($row);
        $row['NRINSJURFORNINI1']     =  $row['NRINSJURFORNINI'];
        $row['NRINSJURFORNFIN1']     =  $row['NRINSJURFORNFIN'];
        $row['CDFORNECEDINI1']       =  $row['CDFORNECEDINI'];
        $row['CDFORNECEDFIN1']       =  $row['CDFORNECEDFIN'];
        $row['NMRAZSOCFORNINI1']     =  $row['NMRAZSOCFORNINI'];
        $row['NMRAZSOCFORNFIN1']     =  $row['NMRAZSOCFORNFIN'];
        $row['IDTPIJURFORNINI1']     =  $row['IDTPIJURFORNINI'];
        $row['IDTPIJURFORNFIN1']     =  $row['IDTPIJURFORNFIN'];
        $row['NRORG1']               =  $row['NRORG'];
        
        $this->reportMapping[$this->stNomeRel] =
            array (
            'strategy' => $row['STRATEGY'],
            'parameterMapping' =>
            array (
                "NRINSJURFORNINI"  => "P_NRINSJURFORNINI",
                "NRINSJURFORNFIN"  => "P_NRINSJURFORNFIN",
                "NMRAZSOCFORNINI"  => "P_NMRAZSOCFORNINI",
                "NMRAZSOCFORNFIN"  => "P_NMRAZSOCFORNFIN",
                "IDTPIJURFORNINI"  => "P_IDTPIJURFORNINI",
                "IDTPIJURFORNFIN"  => "P_IDTPIJURFORNFIN",
                "NRORG"            => "P_NRORG",
                "CDFORNECEDINI"    => "P_CDFORNECEDINI",
                "CDFORNECEDFIN"    => "P_CDFORNECEDFIN",
            ),
        );
    }
      public function verificaPrazo(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $result = $this->service->verificaPrazo($row);
            $response->addDataSet(new DataSet("data", $result));
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
}